#!/bin/sh -
# Copyright (c) 2004 Jean Milanez Melo 
#		     Patrick Tracanelli
#
# $FreeBSD
#set -xv
CURRENTDIR=/usr/src/release/tinybsd
WORKDIR=/usr/obj/tinybsdbuild
KERNCONF=TINYBSD
DEFINSTARGS="-o 0 -g 0 -m 555"
SECTUNIT=$1
TRACKCYL=$2 
SECTRACK=$3
IMG=$4

if [ "$#" -lt 3 ]
then
  echo "Woops!
   Usage: $0 <mediasize in sectors> <heads according to firmware> 
   <sectors according to firmware> [<tinybsd image name>]

   Example: $0 62592 4 32

   Run diskinfo(8) -v against your CF device to get correct information
   about your disk."
  exit 1
fi

check_alt_imgname() {
	if [ -z ${IMG} ]
	then
		IMG="tinybsd.bin"
		echo "Alternative image name not set; defaulting to 'tinybsd.bin'"
	fi
}

remove_workdir() {
	chflags -R noschg ${WORKDIR}
	echo -n "Removing "${WORKDIR}
	rm -rf ${WORKDIR}
	echo " done."
}


prework() {
	remove_workdir	
	mkdir -p ${WORKDIR}
}


create_tree() {
	echo "Creating directory hierarchy... "
	mtree -deU -f /etc/mtree/BSD.root.dist -p ${WORKDIR}
	mtree -deU -f /etc/mtree/BSD.usr.dist -p ${WORKDIR}/usr
	mtree -deU -f /etc/mtree/BSD.var.dist -p ${WORKDIR}/var
}


copy_binaries() {
set -xv
	for file in `cat ${CURRENTDIR}/tinybsd.basefiles | grep -v "#" | \
		cut -f1 -d":" | uniq` ; do
		echo "Copying "/${file}" to "${WORKDIR}/${file} 
		cp -fp /${file} ${WORKDIR}/${file} ;
	done
set +xv
}


make_kernel() {
	echo "Building customized tiny beastie kernel... "
	cp -p ${CURRENTDIR}/${KERNCONF} /usr/src/sys/i386/conf
	cd /usr/src/sys/i386/conf
	cd /usr/src
	make buildkernel KERNCONF=${KERNCONF}
	gzip -9 /usr/src/sys/i386/compile/${KERNCONF}/kernel
	install ${DEFINSTARGS} /usr/src/sys/i386/compile/${KERNCONF}/kernel.gz $WORKDIR/boot/kernel/
	install ${DEFINSTARGS} \
	  /usr/src/sys/i386/compile/${KERNCONF}/modules/usr/src/sys/modules/acpi/acpi/acpi.ko \
	    $WORKDIR/boot/kernel
	install -o 0 -g 0 -m 444 /sys/i386/conf/GENERIC.hints $WORKDIR/boot/device.hints
}


copy_libraries() {
set -xv
	TDEPFILE="`mktemp -t deps`"

	cd ${CURRENTDIR}
	for file in `cat tinybsd.basefiles | grep -v "#" | cut -f1 -d":"`; do
		ldd -f "%p\n" /${file} >> ${TDEPFILE} ; # don't worry on progs been "not dynamic"
	done
	for lib in `cat ${TDEPFILE} | uniq`; do
		echo "Copying "${lib}" to "${WORKDIR}${lib}
		cp -fp ${lib} ${WORKDIR}${lib} ;
	done
        cp -p /usr/lib/pam* ${WORKDIR}/usr/lib
	echo -n "Unlinking "
	rm -fv ${TDEPFILE}
set +xv
}


create_etc() {
	cd /usr/src/etc/sendmail/
	make freebsd.cf freebsd.submit.cf

	cd /usr/src/etc/
	make distribution DESTDIR=${WORKDIR}

	echo "/dev/ad0s1a / ufs ro 1 1" >  $WORKDIR/etc/fstab

cat >> ${WORKDIR}/etc/rc.conf << EORCCONF
hostname="tinybsd.freebsd.org"
sendmail_enable="NONE"
sshd_enable="YES"
usbd_enable="NO"
inetd_enable="NO"
portmap_enable="NO"
update_motd="NO"
varsize="8192"
diskless_mount="/etc/rc.d/diskless"
EORCCONF
}


symlinks() {
set -xv
	for i in `cat tinybsd.basefiles | grep -v "#" | grep ":"`; do
		SOURCE_FILE=`echo $i | awk -F ":" {'print $1'}`
		TARGET_FILE=`echo $i | awk -F ":" {'print $2'}`
		chroot ${WORKDIR} /bin/ln -vs /${SOURCE_FILE} ${TARGET_FILE}
	done
set +xv
}


create_image() {

	set -ex

	VNODEFILE=`mktemp -t tinybsd`
	IMGMNT=`mktemp -d -t tinybsd`

	dd if=/dev/zero of=${VNODEFILE} count=${SECTUNIT}

	MD=`mdconfig -a -t vnode -f ${VNODEFILE} -x ${SECTRACK} -y ${TRACKCYL}`

	diskinfo -v /dev/${MD}

	fdisk -I /dev/${MD}
	fdisk /dev/${MD}

	boot0cfg -B -b ${WORKDIR}/boot/boot0 -s 1 -m 1 ${MD}
	bsdlabel -w -B /dev/${MD}s1
	newfs -O2 -U /dev/${MD}s1a

	mount /dev/${MD}s1a ${IMGMNT}

	( cd ${WORKDIR} && find . -print | cpio -dump ${IMGMNT} ) || true

	df ${IMGMNT}
	umount ${IMGMNT}

	dd if=/dev/${MD} of=${CURRENTDIR}/${IMG} bs=64k

	rm -vf ${VNODEFILE}
	rm -rvf ${IMGMNT}
	mdconfig -d -u ${MD}
}

##########################
# run things
prework
check_alt_imgname
create_tree
copy_binaries
make_kernel
copy_libraries
symlinks
create_etc
create_image
#set +xv

